import os
from contextlib import contextmanager

@contextmanager
def change_dir(destination):
    try:
        cwd = os.getcwd()
        os.chdir(destination)
        yield
    finally:
        os.chdir(cwd)


from subprocess import CalledProcessError

from django.core.management.base import BaseCommand, CommandError

from dbbackup_git.implementation import add_db_backup
from django.conf import settings

class Command(BaseCommand):
    help = f"Update database backup in JSON format, commit and push it with Git"

    def add_arguments(self, parser):
        parser.add_argument(
            '--silent', action='store_true', dest='silent',
            help="Do not show success message",
        )

    def handle(self, **options):
        assert hasattr(settings, 'DBBACKUP_GIT'), '`dbbackup_git` app requires the `DBBACKUP_GIT` setting.'
        assert settings.DBBACKUP_GIT.get('DATABASE_BACKUP_FILENAME', None), \
            "`dbbackup_git` app requires the `DBBACKUP_GIT['DATABASE_BACKUP_FILENAME']` setting."

        try:
            created = add_db_backup(options['silent'])
            if not options['silent']:
                message = 'Backup was created successfully' if created else 'No changes detected'
                self.stdout.write(self.style.SUCCESS(message))
        except CalledProcessError as e:
            output = f'{e.stdout}\n{e.stderr}'
            self.stderr.write(self.style.ERROR(f'ERROR: {e}\nOUTPUT: {output}'))



import os
import subprocess

from django.core.management import call_command
from django.conf import settings

from .utils import change_dir



def add_db_backup(silent=False) -> bool:
    BACKUP_FILENAME = settings.DBBACKUP_GIT['DATABASE_BACKUP_FILENAME']
    do_push = settings.DBBACKUP_GIT.get('PUSH', True)

    with change_dir(os.path.dirname(BACKUP_FILENAME)):
        if do_push:
            run_command('git', 'pull')
        create_backup(BACKUP_FILENAME, silent)
        if run_command('git', 'status', '--porcelain').stdout.strip():
            run_command('git', 'add', '.')
            run_command('git', 'commit', '-m', 'Update database backup')
            if do_push:
                run_command('git', 'push')
            return True
        return False


def run_command(*command: str):
    return subprocess.run(command, check=True, capture_output=True, text=True)


def create_backup(filename, silent):
    verbosity = 0 if silent else 1
    call_command('dumpdata', indent=2, verbosity=verbosity, output=filename)
